﻿using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading.Tasks;

using TestCommon;

namespace TestClient
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region HTTP 클라이언트 구하기 - GetHTTPClient(baseAddress)

        /// <summary>
        /// HTTP 클라이언트 구하기
        /// </summary>
        /// <param name="baseAddress">기준 주소</param>
        /// <returns>HTTP 클라이언트</returns>
        private static HttpClient GetHTTPClient(string baseAddress)
        {
            HttpClient client = new HttpClient();

            client.BaseAddress = new Uri(baseAddress);

            client.DefaultRequestHeaders.Accept.Clear();

            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

            return client;
        }

        #endregion

        #region 제품 추가하기 - AddProduct(client, product)

        /// <summary>
        /// 제품 추가하기
        /// </summary>
        /// <param name="client">클라이언트</param>
        /// <param name="product">제품</param>
        /// <returns>결과 코드 태스크</returns>
        private static async Task<string> AddProduct(HttpClient client, Product product)
        {
            string url = string.Format("api/products/");

            var response = await client.PostAsJsonAsync<Product>(url, product);

            if(response.IsSuccessStatusCode)
            {
                string resultCode = await response.Content.ReadAsAsync<string>();

                return resultCode;
            }

            return null;
        }

        #endregion
        #region 제품 수정하기 - UpdateProduct(client, product)

        /// <summary>
        /// 제품 수정하기
        /// </summary>
        /// <param name="client">클라이언트</param>
        /// <param name="product">제품</param>
        /// <returns>결과 코드 태스크</returns>
        private static async Task<string> UpdateProduct(HttpClient client, Product product)
        {
            string url = string.Format("api/products/");

            var response = await client.PutAsJsonAsync(url, product);

            if(response.IsSuccessStatusCode)
            {
                string resultCode = await response.Content.ReadAsAsync<string>();

                return resultCode;
            }

            return null;
        }

        #endregion
        #region 제품 삭제하기 - DeleteProduct(client, id)

        /// <summary>
        /// 제품 삭제하기
        /// </summary>
        /// <param name="client">클라이언트</param>
        /// <param name="id">ID</param>
        /// <returns>결과 코드</returns>
        private static async Task<string> DeleteProduct(HttpClient client, int id)
        {
            string url = string.Format("api/products/{0}", id);

            var response = await client.DeleteAsync(url);

            if(response.IsSuccessStatusCode)
            {
                string resultCode = await response.Content.ReadAsAsync<string>();

                return resultCode;
            }

            return null;
        }

        #endregion

        #region 모든 제품 리스트 구하기 - GetAllProductList(client)

        /// <summary>
        /// 모든 제품 리스트 구하기
        /// </summary>
        /// <param name="client">클라이언트</param>
        /// <returns>제품 리스트</returns>
        private static async Task<List<Product>> GetAllProductList(HttpClient client)
        {
            string url = string.Format("api/products/");

            HttpResponseMessage response = await client.GetAsync(url);

            if(response.IsSuccessStatusCode)
            {
                List<Product> productList = await response.Content.ReadAsAsync<List<Product>>();

                return productList;
            }

            return null;
        }

        #endregion
        #region 카테고리로 제품 리스트 구하기 - GetProductListByCategory(client, category)

        /// <summary>
        /// 카테고리로 제품 리스트 구하기
        /// </summary>
        /// <param name="client">클라이언트</param>
        /// <param name="category">카테고리</param>
        /// <returns>제품 리스트</returns>
        private static async Task<List<Product>> GetProductListByCategory(HttpClient client, string category)
        {
            string url = string.Format("api/products/?category={0}", category);

            HttpResponseMessage response = await client.GetAsync(url);

            if(response.IsSuccessStatusCode)
            {
                List<Product> productList = await response.Content.ReadAsAsync<List<Product>>();

                return productList;
            }

            return null;
        }

        #endregion
        #region ID로 제품 구하기 - GetProductByID(client, id)

        /// <summary>
        /// ID로 제품 구하기
        /// </summary>
        /// <param name="client">클라이언트</param>
        /// <param name="id">ID</param>
        /// <returns>제품 태스크</returns>
        private static async Task<Product> GetProductByID(HttpClient client, int id)
        {
            string url = string.Format("api/products/{0}", id);

            HttpResponseMessage response = await client.GetAsync(url);

            if(response.IsSuccessStatusCode)
            {
                Product product = await response.Content.ReadAsAsync<Product>();

                return product;
            }

            return null;
        }

        #endregion

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        private static void Main()
        {
            List<Product> productList = null;

            Product product = null;

            using(HttpClient client = GetHTTPClient("http://localhost:8080/"))
            {
                Console.WriteLine("제품을 추가합니다.");
                Console.WriteLine("--------------------------------------------------");

                Product source = new Product();

                source.ID       = 4;
                source.Name     = "제품1";
                source.Price    = 10m;
                source.Category = "그룹1";

                AddProduct(client, source).GetAwaiter().GetResult();

                Console.WriteLine("");
                Console.WriteLine("제품을 수정합니다.");
                Console.WriteLine("--------------------------------------------------");

                source.Name     = "제품2";
                source.Category = "그룹2";

                UpdateProduct(client, source).GetAwaiter().GetResult();

                Console.WriteLine("");
                Console.WriteLine("모든 제품을 조회합니다.");
                Console.WriteLine("--------------------------------------------------");

                productList = GetAllProductList(client).GetAwaiter().GetResult();

                for(int i = 0; i < productList.Count; i++)
                {
                    product = productList[i];

                    Console.WriteLine("{0}, {1}, {2}, {3}", product.ID, product.Name, product.Category, product.Price);
                }

                Console.WriteLine("");
                Console.WriteLine("제품을 삭제합니다.");
                Console.WriteLine("--------------------------------------------------");

                DeleteProduct(client, source.ID).GetAwaiter().GetResult();

                Console.WriteLine("");
                Console.WriteLine("모든 제품을 조회합니다.");
                Console.WriteLine("--------------------------------------------------");

                productList = GetAllProductList(client).GetAwaiter().GetResult();

                for(int i = 0; i < productList.Count; i++)
                {
                    product = productList[i];

                    Console.WriteLine("{0}, {1}, {2}, {3}", product.ID, product.Name, product.Category, product.Price);
                }

                Console.WriteLine("");
                Console.WriteLine("특정 제품을 조회합니다.");
                Console.WriteLine("--------------------------------------------------");

                product = GetProductByID(client, 3).GetAwaiter().GetResult();

                Console.WriteLine("ID       : {0}", product.ID      );
                Console.WriteLine("제품명   : {0}", product.Name    );
                Console.WriteLine("가격     : {0}", product.Price   );
                Console.WriteLine("카테고리 : {0}", product.Category);

                Console.WriteLine("");
                Console.WriteLine("카테고리로 제품 리스트를 조회합니다.");
                Console.WriteLine("--------------------------------------------------");

                productList = GetProductListByCategory(client, "Hardware").GetAwaiter().GetResult();

                for(int i = 0; i < productList.Count; i++)
                {
                    product = productList[i];

                    Console.WriteLine("{0}, {1}, {2}, {3}", product.ID, product.Name, product.Category, product.Price);
                }

                Console.WriteLine("--------------------------------------------------");

                Console.WriteLine("아무 키나 눌러 주시기 바랍니다.");

                Console.ReadKey(true);
            }
        }

        #endregion
    }
}